# Filename: asm/bin/asmd.pl

# Basic perl web server script for Account Services Manager.
# There shouldn't be anything too fancy in here. I want to
# keep this script as plain and featureless as possible, to
# avoid security problems.

use Socket qw(:DEFAULT :crlf);

# Global constants
my $default  = 'index.cgi';
my $success  = 'HTTP/1.0 200 OK';
my $logging  = 1;
my $perl     = $^X;
my $base_dir = $ARGV[0];

# Setup the response codes.
my @responses;
$responses[200] = 'OK';
$responses[400] = 'Method Not Allowed';
$responses[401] = 'Access Denied';
$responses[403] = 'Forbidden';
$responses[404] = 'Not Found';
$responses[408] = 'Server Timeout';
$responses[500] = 'Internal Server Error';

# Declare some global variables.
my $remote_ip;
my $request_method;
my $request_uri;
my $request_protocol;
my $file_name;
my $request_args;
my $post_data;
my %new_environ;
my $extra_headers;

# Move into the ASM directory specified on the command line.
unless(chdir($base_dir)) {
    exit;
}

# Setup file descriptors.
close(STDERR);
open(STDERR, ">>logs/error.log");
open(SOCKET, "+>&STDIN");
select(SOCKET);
$| = 1;
select(STDERR);
$| = 1;
select(STDOUT);

# Obtain the IP address of the client.
$remote_ip = inet_ntoa((unpack_sockaddr_in(getpeername(SOCKET)))[1]); 

# Now, read in the request from the client (there is a 15-second timeout).
my $finished_request = 0;
vec($rin, fileno(SOCKET), 1) = 1;
select($rin, undef, undef, 15) || exit();
while(<SOCKET>) {
    $_ =~ s/\r//g;

    if($_ =~ /^(GET|POST)\s*(.*)\s*HTTP\/(.*)\n$/i) {
        $request_method   = $1;
        $request_uri      = $2;
        $request_protocol = $3;
    }
    elsif($_ eq "\n") {
        $finished_request = 1;
        last;
    }
    else {
        my @tmp = split(/: /);
        $tmp[1] =~ s/\n$//;
        $extra_headers{$tmp[0]} = $tmp[1];
    }
}

# Looks like the user disconnected before finishing the request.
if(!$finished_request) {
    shutdown(SOCKET, 2);
    exit;
}

if($request_method eq 'POST') {
    if(exists $extra_headers{'Content-Length'}) {
        if(!read(SOCKET, $post_data, $extra_headers{'Content-Length'})) {
            print STDERR "Unable to read post data: $!\n";
            error(500);
        }
    }
    else {
        error(400);
    }
}

# Parse the request.
if($request_uri =~ /\.\.\// || ($request_method ne 'GET' && $request_method ne 'POST')) {
    error(400);
}
else {
    $request_uri =~ s/\s$//;
    if($request_uri eq '/' || !$request_uri) {
        $request_method = 'GET';
        $request_uri    = '/' . $default;
    }
}

if($request_method eq 'GET') {
    my @tmp       = split(/\?/, $request_uri);
    $file_name    = substr($tmp[0], 1);
    $request_args = $tmp[1];
}
else {
    $file_name = substr($request_uri, 1);
}

# Setup enviroment variables.
$new_env{'PATH'}              = $ENV{'PATH'};
$new_env{'SCRIPT_NAME'}       = $file_name;
$new_env{'SCRIPTFILENAME'}    = "$base_dir/cgi/$file_name";
$new_env{'QUERY_STRING'}      = $request_args;
$new_env{'REQUEST_METHOD'}    = $request_method;
$new_env{'REQUEST_URI'}       = $request_uri;
$new_env{'HTTP_COOKIE'}       = $extra_headers{'Cookie'};
$new_env{'HTTP_REFERRER'}     = $extra_headers{'Referrer'};
$new_env{'HTTP_HOST'}         = $extra_headers{'Host'};
$new_env{'SERVER_NAME'}       = $extra_headers{'Host'};
$new_env{'REMOTE_ADDR'}       = $remote_ip;
$new_env{'REMOTE_HOST'}       = $remote_ip;
$new_env{'SERVER_PROTOCOL'}   = 'HTTP/1.0';
$new_env{'GATEWAY_INTERFACE'} = 'CGI/1.1';
$new_env{'DOCUMENT_ROOT'}     = "$base_dir/cgi";
$new_env{'CONTENT_LENGTH'}    = $extra_headers{'Content-Length'};

# We should have all the information at this point. Check for the file and process it accordingly.
if(-e "cgi/$file_name") {
    my $extension = (split(/\./, $file_name))[1];

    if($extension eq 'jpg' || $extension eq 'gif' || $extension eq 'jpeg') {
        my $buffer;
        my($size, $last_mod) = (stat("cgi/$file_name"))[7, 9];
        send_data("$success\r\n");
        send_data("Accept-Ranges: bytes\r\n");
        send_data("Content-Length: $size\r\n");
        send_data("Date: " . getgmtime(time()) . "\r\n");
        send_data("Last-Modified: " . getgmtime($last_mod) . "\r\n");
        send_data("Server: Account Services Manager\r\n");
        if($extension eq 'jpg' || $extension eq 'jpeg') {
            send_data("Content-type: image/jpeg\r\n");
        }
        else {
            send_data("Content-type: image/gif\r\n");
        }
        send_data("\r\n");
        open(PIC, "cgi/$file_name");
        while(read(PIC, $buffer, 1024) > 0) {
            send_data($buffer);
        }
        close(PIC);
        log_response("HTTP 200 $request_method $request_uri");
    }
    elsif($extension eq 'txt' || $extension eq 'html') {
        open(TXT, "cgi/$file_name");
        my $buffer = join('', <TXT>);
        close(TXT);
        send_data("$success\015\012Content-type: text/plain\015\012\015\012$buffer");
        log_response("HTTP 200 $request_method $request_uri");
    }
    elsif($extension eq 'cgi') {
        pipe(MASTER_IN,  SLAVE_IN);
        pipe(MASTER_OUT, SLAVE_OUT);
        pipe(MASTER_ERR, SLAVE_ERR);

        my $pid = fork();

        if($pid) {
            close(MASTER_IN);
            close(SLAVE_OUT);
            close(SLAVE_ERR);

            select(MASTER_OUT);
            $| = 1;
            select(SLAVE_IN);
            $| = 1;
            select(STDOUT);

            if($request_method eq 'POST') {
                print SLAVE_IN $post_data;
            }

            my $buffer;
            my $headers_done = 0;

            my($line, $line2);
            while(1) {
               $line  = <MASTER_OUT>;
               $line2 = <MASTER_ERR>;

               if(!$line) {
                   last;
               }

               if($line2) {
                   print STDERR $line2;
               }

               if($headers_done) {
                   if(length($line) <= 1024) {
                       send_data($line);
                   }
                   else {
                       # We have a data chunk larger than 1 KB. Split it up into 32-byte chunks and output them separately.
                       my @bytes = split(//, $line);
                       my @streams;
                       my($i, $j) = 0;
                       foreach(@bytes) {
                           if($i / 32 == $j) {
                               $j++;
                           }
                           $streams[$j] .= $bytes[$i];
                           $i++;
                       }

                       foreach(@streams) {
                           send_data($_);
                       }
                   }
               }
               else {
                   $buffer .= $line;
                   if($line eq "\n") {
                       $headers_done = 1;
                       send_data("$success\n$buffer");
                       undef $buffer;
                   }
                   elsif($line =~ /^Location: (.*)\n$/) {
                       send_data("HTTP/1.0 302 Moved Temporarily\nLocation: $1\n");
                       send_data($buffer);
                       close(MASTER_OUT);
                       close(MASTER_ERR);
                       shutdown(SOCKET, 1);
                       exit;
                   }
               }
            }

            close(CGI);

            if(!$headers_done) {
                print STDERR "Script $file_name did not return a complete set of HTTP headers.\n";
                error(500);
            }
            else {
                log_response("HTTP 200 $request_method $request_uri");
            }
        }
        elsif($pid == 0) {
            chdir('cgi');
            %ENV = %new_env;

            open(STDIN,  "<&MASTER_IN");
            open(STDOUT, ">&SLAVE_OUT");
            open(STDERR, ">&SLAVE_ERR");

            close(MASTER_IN);
            close(SLAVE_OUT);
            close(SLAVE_ERR);

            exec($perl, $file_name);
            die "Unable to exec $perl $file_name\n";
        }
        elsif($pid < 0) {
            print STDERR "Unable to fork child process for $file_name.\n";
            error(500);
        }
    }
    else {
        print STDERR "Unknown file type $extension for $file_name.\n";
        error(403);
    }
}
else {
    error(404);
}

sub send_data
{
    syswrite(SOCKET, $_[0]);
}

sub error
{
    send_data("HTTP/1.0 $_[0] $responses[$_[0]]$CRLF");
    send_data("Server: Account Services Manager$CRLF");
    send_data("Connection: Close$CRLF");
    send_data("Date: " . getgmtime(time()) . $CRLF);
    send_data("Content-type: text/html$CRLF$CRLF");
    send_data("<html>\015\012<head>\015\012<title>Error $_[0]</title>\015\012</head>\015\012\015\012<body bgcolor=\"#FFFFFF\" textcolor=\"#000000\" linkcolor=\"#000000\" vlink=\"#000000\">\015\012");
    send_data("Error $_[0]\015\012</body>\015\012</html>\015\012");
    shutdown(SOCKET, 2);
    log_response("HTTP $_[0] $request_method $request_uri");
    exit;
}

sub log_response
{
    if($logging) {
        open(LOG, ">>$base_dir/logs/asm.log");
        my $date = localtime(time);
        print LOG "[$date $remote_ip $user_agent] $_[0]\n";
        close(LOG);
    }
}

sub getgmtime
{
    my @t = gmtime($_[0]);

    my @days = (
        "Sun",
        "Mon",
        "Tue",
        "Wed",
        "Thu",
        "Fri",
        "Sat"
    );

    my @months = (
        "Jan",
        "Feb",
        "Mar",
        "Apr",
        "May",
        "Jun",
        "Jul",
        "Aug",
        "Sep",
        "Oct",
        "Nov",
        "Dec"
    );

    my $date;
    $date  = $days[$t[6]];
    $date .= ", ";
    $date .= sprintf("%2d", $mday + 1);
    $date .= " ";
    $date .= $months[$t[4]];
    $date .= " ";
    $date .= $t[5] + 1900;
    $date .= " ";
    $date .= sprintf("%2d", $t[2]);
    $date .= ":";
    $date .= sprintf("%2d", $t[1]);
    $date .= ":";
    $date .= sprintf("%2d", $t[0]);
    $date .= " GMT";

    return $date;
}
