###
### Account Services Manager API Library
###
########################################################################
### This program is free software; you can redistribute it and/or
### modify it under the terms of the GNU General Public License
### as published by the Free Software Foundation; either version 2
### of the License, or (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
########################################################################

# Constants for logging levels.
use constant LOG_LEVEL_DEBUG => 1;
use constant LOG_LEVEL_INFO  => 2;
use constant LOG_LEVEL_WARN  => 3;
use constant LOG_LEVEL_CRIT  => 4;
use constant LOG_LEVEL_FATAL => 5;

# Put everything in here into its own namespace.
package API;

# Private Library variables.
my($cwd);

# Load a specified library into this library's namespace, with extensive error-checking.
# We use do() rather than require() so that a poorly written library can't crash the program.
sub LoadLibrary
{
    my($file) = @_;

    my $lib;
    unless($lib = do $file)
    {
        if(!defined($lib))
        {
            LogError($WARNING, "Couldn't load $file: $!");
            return 0;
        }
        if($@)
        {
            LogError($WARNING, "Parse error in $file: $@");
            return 0;
        }
        if(!$lib)
        {
            LogError($WARNING, "Couldn't load $file. Check if it's missing a 1;.");
            return 0;
        }
    }
    return 1;
}

# Update the user config file with data modified during run-time.
sub UpdateUserConf
{
    open(USERCONF, ">$::BaseDir/users/$AUTH::User") || &::FatalError("Could not update user configuration file: $!");
    foreach(keys %::UserConf)
    {
        print USERCONF $_, ": ", $::UserConf{$_}, "\n";
    }
    close USERCONF || &::FatalError("Failed updating user configuration file in API::UpdateUserConf(): $!");
}

# Log an action in the local log file.
sub LogAction
{
    my $time  = localtime;
    my($text) = @_;

    open(LOG, ">>$::BaseDir/logs/asm.log") || return 0;
    print LOG "[$time $ENV{'REMOTE_ADDR'} $AUTH::User] $::module: $text\n";
    close LOG || return 1;

    return 1;
}

# Log an error to the error log
sub LogError
{
    my($log_level, $error) = @_;

    if(exists $::Global_Config{'MinErrorLogLevel'})
    {
        return 1 if($log_level < $::Global_Config{'MinErrorLogLevel'}[1]);
    }

    my $time = localtime;

    open(LOG, ">>$::BaseDir/logs/error.log") || return 0;
    print LOG "[$time $ENV{'REMOTE_ADDR'} $AUTH::User] $error\n";
    close LOG || return 0;

    return 1;
}

# Load the authentication module as per the configuration file.
sub LoadAuthModule
{
    do "$::BaseDir/lib/auth/$::Global_Config{'AuthModule'}[1].pl";
}

# Generate a random encryption salt.
sub RandomSalt
{
    return join('', ('A' .. 'Z', 0 .. 9, 'a' .. 'z', '.', '/')[rand 64, rand 64]);
}

# What directory is COMMAND in?
sub which
{
    my @dirs = split(/:/, $ENV{'PATH'});
    foreach(@dirs) { return $_ if(-e "$_/$_[0]") }
    return "";
}

# Is a character in an array?
sub is_in_array
{
    foreach(@_) { return 1 if($_ eq $_[0]) }
    return 0;
}

# Get the base directory path to a file.
sub BaseDir
{
    my @path = split('/', $_[0]);
    return if !$path[0];

    splice(@path, $#path - 2, 1);
    return @path;
}

# Change the current working directory. It's just a wrapper for chdir().
sub change_dir
{
    my($cd) = @_;

    $cwd = $cd;
    return chdir($cd);
}

# Get the current directory. This is the reason for change_dir() above.
sub getcwd
{
    return $cwd;
}

# Take a directory listing as an array, and remove everything except regular directories.
sub filterdirs
{
    my($directory) = @_;
    my @ret;

    my $count = 0;
    foreach(@_)
    {
        if(!(/^\./ || !-d "$directory/$_"))
        {
            $ret[$count] = "$directory/$_";
            $count++;
        }
    }
    return @ret;
}

# Get a recursive list of all subdirectories under DIR.
sub RecursiveList
{
    my $d = getcwd();
    return if(!change_dir(shift));

    opendir(HOME, '.');
    my @dirs = readdir(HOME);
    closedir HOME;
    @dirs = filterdirs('.', @dirs);
 
    my @tmp_arr;
    foreach(@dirs)
    {
        $_ =~ s/^\.\///;
        opendir(SUBDIR, $_);
        @tmp_arr = filterdirs($_, readdir(SUBDIR));
        push(@dirs, @tmp_arr);
        closedir(SUBDIR);
    }
    change_dir($d);
    return @dirs;
}

# RecursiveDelete() - Recursively deletes a directory.
sub RecursiveDelete
{
    my $dir = @_;
    my @list;
    my $errors = 0;

    # It's just a file, delete it and exit.
    return(unlink($dir) != 1) if(!-d $dir);

    # Get a list of what's in here.
    opendir(DIR, $dir) || return 0;
    @list = readdir(DIR);
    closedir(DIR);

    foreach(@list)
    {
        next if($_ eq '.' || $_ eq '..');
        if(-d $_) {
            my $ret = RecursiveDelete("$dir/$_");
            rmdir("$dir/$_") unless($errors);
            $errors += $ret;
        }
        else {
            $errors++ if(unlink($_) != 1);
        }
    }

    return $errors;
}

# Print the contents of FILE to HANDLE
sub copy_file_to_handle
{
    my($file, $handle) = @_;

    (!$handle) and $handle = STDOUT;

    print $handle (fread($file));
}

# Return an array of characters from FILE.
sub file
{
    my($file) = @_;
    
    open(FILE, $file);
    my @ret = <FILE>; 
    close FILE;
    
    return @ret;
}
 
# Get a line from a file without the newline
sub GetLineFromFile
{
    my($file) = @_;
    
    open(FILE, $file);
    my $text = <FILE>;
    close FILE;
    
    chop $text;
    return $text;
}

# Is a particular file or directory in perl's include path?
sub InPerlINC
{
    my($file) = @_;
    $file =~ s/::/\//g;

    foreach(@INC)
    {
        return 1 if(-e "$_/$file");
    }
    
    return 0;   
}
    
# Return the text of a cookie. Returns an array of cookies if none specified.
sub GetCookie
{
    my($name) = @_;
    my @cookies = split(/;\s?/, $ENV{'HTTP_COOKIE'});
    return @cookies if(!$name);

    foreach(@cookies)
    {
        return (split(/=/))[1] if($_ =~ /^$name=/);
    }
}

sub GetCookies
{
    return GetCookie();
}

# Change the user IDs (optionally include a gid)
sub setuid
{
    my($uid, $gid) = @_;

    $< = $uid;
    $> = $uid;

    if(length($gid) > 0)
    {
        setgid($gid);
    }

    return($< == $uid && $> == $uid);
}

# Change the group IDs
sub setgid
{
    my($gid) = @_;
    $( = $gid;
    $) = $gid;

    return($( == $gid && $) == $gid);
}

# SQL Configuration section.
my($sql_host, $sql_db, $sql_user, $sql_pass);

sub LoadSQL_Config
{
    open(SQL, "$::BaseDir/conf/sql-auth.conf");
    my @lines = split("\n", join('', <SQL>));
    close SQL;

    my @tmp;

    foreach(@lines) {
        @tmp = split(/\s/);
        if($tmp[0] eq 'Hostname') {
            $sql_host = $tmp[1];
        }
        elsif($tmp[0] eq 'Database') {
            $sql_db = $tmp[1];
        }
        elsif($tmp[0] eq 'Username') {
            $sql_user = $tmp[1];
        }
        elsif($tmp[0] eq 'Password') {
            $sql_pass = $tmp[1];
        }
    }
}

sub GetSQLHostname
{
    return $sql_host;
}

sub GetSQLDatabase
{
    return $sql_db;
}

sub GetSQLUsername
{
    return $sql_user;
}

sub GetSQLPassword
{
    return $sql_pass;
}

1;
